package se.chalmers.paragon;

import java.util.HashMap;
import java.util.LinkedList;

public class Util {

	public static Object[] concat(Object[] a, Object[] b) {
		Object[] res = new Object[a.length + b.length];
		System.arraycopy(a, 0, res, 0, a.length);
		System.arraycopy(b, 0, res, a.length, b.length);
		return res;
	}

	/**
	 * Extends the mapping from actor variables to concrete actors
	 * 
	 * @param varmap
	 *            Current hashmap
	 * @param from
	 *            LockPair containing actor variables
	 * @param to
	 *            The concrete actors either matching the concrete actors in the
	 *            provided lock pair or being the substitute concrete actors for
	 *            the actor variables
	 * @return A new hashmap with the additional mapping added
	 */
	public static HashMap<ActorVariable, ConcreteActor> extendVarMap(
			HashMap<ActorVariable, ConcreteActor> varmap, Atom from,
			ActorList to) {
		HashMap<ActorVariable, ConcreteActor> res = new HashMap<ActorVariable, ConcreteActor>();
		res.putAll(varmap);
		for (int i = 0; i < from.getArgs().getActors().length; i++) {
			if (from.getArgs().getActors()[i] instanceof ActorVariable)
				res.put((ActorVariable) from.getArgs().getActors()[i],
						(ConcreteActor) to.getActors()[i]);
		}
		return res;
	}

	/**
	 * Replaces variables in the pair according to the provided mapping
	 * 
	 * @param pair
	 *            A lock instance with variables
	 * @param varmap
	 *            A mapping from variables to concrete actors
	 * @return A pair with variables occurring in the variable mapping replaced
	 *         with the corresponding concrete actors.
	 */
	public static Atom substitute(Atom pair,
			HashMap<ActorVariable, ConcreteActor> varmap) {
		Actor[] current = pair.getArgs().getActors();
		Actor[] args = new Actor[current.length];
		for (int i = 0; i < current.length; i++)
			if (varmap.containsKey(current[i]))
				args[i] = varmap.get(current[i]);
			else
				args[i] = current[i];
		return new Atom(pair.getLock(), args);
	}

	public static Atom substituteVar(Atom pair,
			HashMap<ActorVariable, ActorVariable> varmap) {
		Actor[] current = pair.getArgs().getActors();
		Actor[] args = new Actor[current.length];
		for (int i = 0; i < current.length; i++)
			if (varmap.containsKey(current[i]))
				args[i] = varmap.get(current[i]);
			else
				args[i] = current[i];
		return new Atom(pair.getLock(), args);
	}

	public static HashMap<ActorVariable, ConcreteActor> skolemVars(Atom _l,
			HashMap<ActorVariable, ConcreteActor> varmap) {

		Atom l = Util.substitute(_l, varmap);
		for (Actor actor : l.getArgs().getActors())
			if (actor instanceof ActorVariable) {
				varmap.put((ActorVariable) actor,
						ConcreteActor.freshActor("freshactor"));
				return skolemVars(l, varmap);
			}
		return varmap;
	}

	public static HashMap<ActorVariable, ConcreteActor> sameConcreteActors(
			Atom base, Atom other) {
		int arity = base.getArgs().arity();
		HashMap<ActorVariable, ConcreteActor> map = new HashMap<ActorVariable, ConcreteActor>();
		for (int i = 0; i < arity; i++)
			if (base.getArgs().getActors()[i] instanceof ConcreteActor) {
				if (other.getArgs().getActors()[i] instanceof ConcreteActor)
					if (!base.getArgs().getActors()[i].equals(other.getArgs()
							.getActors()[i]))
						return null;
			} else {
				if (other.getArgs().getActors()[i] instanceof ConcreteActor) {
					map.put((ActorVariable) base.getArgs().getActors()[i],
							(ConcreteActor) other.getArgs().getActors()[i]);
				}
			}
		return map;
	}

	public static Atom[] substitutes(Atom[] pairs,
			HashMap<ActorVariable, ConcreteActor> varmap) {
		Atom[] res = new Atom[pairs.length];
		for (int i = 0; i < pairs.length; i++)
			res[i] = Util.substitute(pairs[i], varmap);
		return res;
	}

	public static Atom[] substitutesVar(Atom[] pairs,
			HashMap<ActorVariable, ActorVariable> varmap) {
		Atom[] res = new Atom[pairs.length];
		for (int i = 0; i < pairs.length; i++)
			res[i] = Util.substituteVar(pairs[i], varmap);
		return res;
	}

	public static int getHighestVar(Atom[] body) {
		int max = -1;
		for (Atom b : body)
			for (Actor a : b.getArgs().getActors())
				if (a instanceof ActorVariable)
					max = Math.max(max, ((ActorVariable) a).getVar());
		return max;
	}

	public static HashMap<ActorVariable, ActorVariable> createVarVarMap(
			Atom thishead, int v) {
		HashMap<ActorVariable, ActorVariable> res = new HashMap<ActorVariable, ActorVariable>();
		Actor[] actors = thishead.getArgs().getActors();
		for (int i = 0; i < actors.length; i++)
			if (actors[i] instanceof ActorVariable) {
				res.put((ActorVariable) actors[i], Actor.newActorVariable(v));
				v++;
			}
		return res;
	}

	public static LinkedList<Clause> copyClause(LinkedList<Clause> clauses,
			Lock to) {
		LinkedList<Clause> res = new LinkedList<Clause>();
		for (Clause c : clauses) {
			Atom[] nbody = new Atom[c.getBody().length];
			for (int i = 0; i < c.getBody().length; i++) {
				Atom p = c.getBody()[i];
				if (p.getLock().equals(c.getHead().getLock()))
					nbody[i] = Atom.newAtom(to, p.getArgs());
				else
					nbody[i] = p;
			}
			res.add(Clause.newClause(Atom.newAtom(to, c.getHead().getArgs()),
					nbody));
		}
		return res;
	}

}
